#define USE_ARDUINO_INTERRUPTS true    // Set-up low-level interrupts for most acurate BPM math
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#include <ezButton.h>
#include <PulseSensorPlayground.h>

// Set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_I2C lcd(0x27,20,4);  

// Define PINS
#define R_LED 3
#define Y_LED 4
#define G_LED 5
#define PULSE_LED 11

#define BUTTON 2
#define BUZZER 12
#define PULSE_PIN 0

ezButton button(BUTTON);  // button variable
bool state = LOW;

PulseSensorPlayground pulseSensor; // pulse sensor variable
int thresh = 510;
int lastBPM = 0;
int maxBPM = 0;
int minBPM = 0;

// polygraph answer: 0 - idk, 1 - true, 2 - lie
int ans = 0;
// polygraph questions
String quest[] = {"Cum te numesti?", "Ce varsta ai?", "Ai vreo restanta?",
                  "Ai copiat la vreun examen?", "Treci la PM?"};

// heart symbol
byte heart[8] = {
  0b00000,
  0b01010,
  0b11111,
  0b11111,
  0b01110,
  0b00100,
  0b00000,
  0b00000
};

void setup() {
  Serial.begin(115200);

  // setup LEDs
  pinMode(R_LED, OUTPUT);
  pinMode(Y_LED, OUTPUT);
  pinMode(G_LED, OUTPUT);

  // setup BUZZER
  pinMode(BUZZER, OUTPUT);
  pinMode(BUTTON, INPUT);

  // setup pulse sensor
  pulseSensor.analogInput(PULSE_PIN);
  pulseSensor.blinkOnPulse(PULSE_LED);
  pulseSensor.setThreshold(thresh);
  pulseSensor.setSerial(Serial);
  pulseSensor.setOutputType(PROCESSING_VISUALIZER);
  pulseSensor.begin();

  // initialize the lcd 
  lcd.init();
  lcd.backlight();

  button.setDebounceTime(50);
}

// Function to test the pulse for the polygraph question
void execute()
{
  // get BPM from pulse sensor
  int myBPM = pulseSensor.getBeatsPerMinute();
  pulseSensor.outputSample();

  // if there was a heartbeat
  if (pulseSensor.sawStartOfBeat()) {
    // print BPM to lcd
    lcd.setCursor(5, 0);
    lcd.print("BPM: ");
    lcd.print(myBPM);
    lcd.createChar(0, heart);
    lcd.setCursor(8, 1);
    lcd.write((byte)0);

    // output heartbeat
    pulseSensor.outputBeat();
    tone(BUZZER, 900);
    
    if (lastBPM == 0) {
      lastBPM = myBPM;
      minBPM = myBPM;
      maxBPM = myBPM;
    }

    // set max and min BPM
    minBPM = min(myBPM, minBPM);
    maxBPM = max(myBPM, maxBPM);
  } else
    noTone(BUZZER);
    
  delay(20);
}

// Function to ask a polygraph question
void ask_question(int nr, String str)
{
  maxBPM = 0;
  minBPM = 0;
  lastBPM = 0;

  // print question number to lcd
  lcd.clear();
  lcd.setCursor(2, 0);
  lcd.print("INTREBAREA ");
  lcd.print(nr);
  delay(3000);

  // print question to lcd
  lcd.clear();
  if (str.length() <= 16) {
    lcd.setCursor((16 - str.length()) / 2, 0);
    lcd.print(str);
  } else {
    int len = 15;
    while (str[len] != ' ')
      len--;
    lcd.setCursor((16 - len) / 2, 0);
    lcd.print(str.substring(0, len));
    lcd.setCursor((16 - str.length() + len) / 2, 1);
    lcd.print(str.substring(len + 1, str.length()));
  }
  
  delay(3000);
  lcd.clear();
  long int curr_time = millis();

  // test pulse for 10s
  while (millis() - curr_time < 10000) {
    execute();
  }

  noTone(BUZZER);
  // get answer depending on min and max BPM
  if (maxBPM - minBPM >= 0 && maxBPM - minBPM <= 5)
    ans = 1;
  else if (maxBPM - minBPM >= 5 && maxBPM - minBPM <= 10)
    ans = 2;
  else
    ans = 0;

  // depending on the answer, turn on LEDs
  lcd.clear();
  switch (ans) {
    case 0:
      // yellow LED for invalid answer
      digitalWrite(Y_LED, HIGH);
      lcd.setCursor(4, 0);
      lcd.print("INVALID");
      break;
    case 1:
      // green LED for true answer
      digitalWrite(G_LED, HIGH);
      lcd.setCursor(4, 0);
      lcd.print("ADEVARAT");
      break;
    case 2:
      // red LED for false answer
      digitalWrite(R_LED, HIGH);
      lcd.setCursor(6, 0);
      lcd.print("FALS");
      break;
  }
  delay(3000);
  
  digitalWrite(R_LED, LOW);
  digitalWrite(Y_LED, LOW);
  digitalWrite(G_LED, LOW);
}

// Function for polygraph execution
void execute_polygraph()
{
  lcd.clear();
  noTone(BUZZER);

  // print starting message
  lcd.setCursor(4, 0);
  lcd.print("POLIGRAF");
  lcd.setCursor(5, 1);
  lcd.print("ACTIV");
  delay(5000);

  // ask polygraph questions
  for (int i = 1; i <= 5; i++)
    ask_question(i, quest[i - 1]);
  
  lcd.clear();
  noTone(BUZZER);
}


void loop() {
  button.loop();

  // when the button is pressed, start the polygraph
  if (button.isPressed())
    execute_polygraph();
  else {
    // time for polygraph setup
    lcd.setCursor(6, 0);
    lcd.print("TEST");
  }
}
